/**
 * Card.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.zip.DataFormatException;

/**
 * <code>Card</code> represents the high-level intuition of a flash card. <code>Card</code> represents a single card in
 * a {@link Deck} and maintains all the features of a real flash-card as well as additional, computer-only
 * possibilities.
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class Card
{
	/** A <code>String</code> representing the question associated with this <code>Card</code> */
	public String question;

	/** A <code>String</code> representing the answer associated with this <code>Card</code> */
	public String answer;

	/** A <code>String</code> representing a hint associated with this <code>Card</code> */
	private String hint;

	/** A <code>String</code> representing additional information associated with this </code>Card</code> */
	private String additionalInfo;

	/** A {@link HashMap} representing the card's options */
	public HashMap<String, Boolean> options;

	/**
	 * Instantiates a new Card
	 */
	public Card()
	{
		this.options = new HashMap<String, Boolean>();
		this.answer = new String();
		this.question = new String();
		this.hint = new String();
		this.additionalInfo = new String();
		setDefaultSettings();
	}

	/**
	 * Instantiates a <code>Card</code> with the specified parameters.
	 * 
	 * @param q
	 *        a <code>String</code> representing the question associated with this <code>Card</code>
	 * @param a
	 *        a <code>String</code> representing the answer associated with this <code>Card</code>
	 * @param h
	 *        a <code>String</code> representing the hint associated with this <code>Card</code>
	 * @param i
	 *        a <code>String</code> representing the additional information associated with this <code>Card</code>
	 */
	public Card(String q, String a, String h, String i)
	{
		this.question = q;
		this.answer = a;
		setHint(h);
		setAdditionalInfo(i);

		this.options = new HashMap<String, Boolean>();
		setDefaultSettings();
	}

	/**
	 * Compares the provided <code>String</code> with this <code>Card</code>'s correct answer
	 * 
	 * @param a
	 *        a <code>String</code> representing the provided answer
	 * @return <code>true</code> if the answer sent matches, <code>false</code> otherwise
	 */
	public boolean checkAnswer(String a)
	{
		return a.equalsIgnoreCase(this.answer.toString());
	}

	/**
	 * Returns the additional information associated with this card
	 * 
	 * @return a <code>String</code> representing the additional information associated with this <code>Card</code>
	 */
	public String getAdditionalInfo()
	{
		return this.additionalInfo;
	}

	/**
	 * Returns the answer associated with this <code>Card</code>
	 * 
	 * @return a <code>String</code> representing this <code>Card</code>'s answer
	 */
	public String getAnswer()
	{
		return this.answer;
	}

	/**
	 * Get this <code>Card</code>'s current hint
	 * 
	 * @return The current hint
	 */
	public String getHint()
	{
		return this.hint;
	}

	/**
	 * Returns this <code>Card</code>'s question
	 * 
	 * @return the question
	 */
	public String getQuestion()
	{
		return this.question.toString();
	}

	/**
	 * Import a <code>Card</code> from disk
	 * 
	 * @param file
	 *        the file to be read from
	 * @throws IOException
	 *         if a problem with the input has occurred
	 * @throws DataFormatException
	 *         if data is found to be in the incorrect format
	 */
	public static Card readFromDisk(BufferedReader file) throws IOException, DataFormatException
	{
		String line = file.readLine().trim();
		if(!(line.contains("<question>") && line.contains("</question>")))
			throw new DataFormatException();
		String question = line.substring(line.indexOf("<question>") + 10, line.lastIndexOf("</question>"));
	
		line = file.readLine().trim();
		if(!(line.contains("<answer>") && line.contains("</answer>")))
			throw new DataFormatException();
		String answer = line.substring(line.indexOf("<answer>") + 8, line.lastIndexOf("</answer>"));
	
		line = file.readLine().trim();
		if(!(line.contains("<hint>") && line.contains("</hint>")))
			throw new DataFormatException();
		String hint = line.substring(line.indexOf("<hint>") + 6, line.lastIndexOf("</hint>"));
	
		line = file.readLine().trim();
		if(!(line.contains("<info>") && line.contains("</info>")))
			throw new DataFormatException();
		String info = line.substring(line.indexOf("<info>") + 6, line.lastIndexOf("</info>"));
	
		Card c = new Card(question, answer, hint, info);
	
		line = file.readLine().trim();
		if(!(line.contains("<options>") && line.contains("</options>")))
			throw new DataFormatException();
		String[] optionArray = line.substring(line.indexOf("<options>") + 10, line.lastIndexOf("}</options>")).split(", ");
		for(String o : optionArray)
		{
			String[] temp = o.split("=");
			String key = temp[0].trim();
			String value = temp[1].trim();
			c.setSetting(key, Boolean.valueOf(value));
		}
	
		if(!file.readLine().trim().equals("</Card>"))
			throw new DataFormatException();
	
		return c;
	}

	/**
	 * Sets the additional information associated with this card to the specified <code>String</code>
	 * 
	 * @param i
	 *        a <code>String<code> representing the card's additional information
	 */
	public void setAdditionalInfo(String i)
	{
		if(i == null)
			this.additionalInfo = "";
		else
			this.additionalInfo = i;
	}

	/**
	 * Set the default <code>Card</code> settings
	 */
	private void setDefaultSettings()
	{
		this.options.clear();
		this.options.put("Display Hints", new Boolean(false));
		this.options.put("Self-check Answers", new Boolean(false));
	}

	/**
	 * Sets the hint associated with this card to the specified <code>String</code>
	 * 
	 * @param h
	 *        a <code>String</code> representing the card's hint
	 */
	public void setHint(String h)
	{
		if(h == null)
			this.hint = "";
		else
			this.hint = h;
	}

	/**
	 * Set the provided setting. Update the setting if it already exists.
	 * 
	 * @param key
	 *        The setting name
	 * @param value
	 *        the Boolean value
	 */
	public void setSetting(String key, Boolean value)
	{
		if(this.options != null)
			this.options.put(key, value);
	}

	/**
	 * Set the values of this <code>Card</code>
	 * 
	 * @param q
	 *        a <code>String</code> representing the question associated with this <code>Card</code>
	 * @param a
	 *        a <code>String</code> representing the answer associated with this <code>Card</code>
	 * @param h
	 *        a <code>String</code> representing the hint associated with this <code>Card</code>
	 * @param i
	 *        a <code>String</code> representing the additional information associated with this <code>Card</code>
	 * @param pref
	 *        A {@link HashMap} representing the options associated with this <code>Card</code>
	 */
	public void setValues(String q, String a, String h, String i, HashMap<String, Boolean> pref)
	{
		this.question = q;
		this.answer = a;
		setHint(h);
		setAdditionalInfo(i);

		for(Entry<String, Boolean> j : pref.entrySet())
			this.options.put(new String(j.getKey()), new Boolean(j.getValue()));
	}

	/**
	 * Writes out the properties of the card
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer b = new StringBuffer();
		b.append("-----");
		b.append("Card:\n");
		b.append("Question: " + this.question + "\n");
		b.append("Answer: " + this.answer + "\n");
		b.append("Additional Info: " + this.additionalInfo + "\n");
		b.append("Hint: " + this.hint + "\n");
		b.append("Settings: \n");

		for(Entry<String, Boolean> i : this.options.entrySet())
			b.append(i.getKey() + ":" + i.getValue().toString() + "\n");
		b.append("-----");
		return b.toString();
	}

	/**
	 * Export the contents of this <code>Card</code> to disk
	 * 
	 * @param file
	 *        the file to be written to
	 * @throws IOException
	 *         if a problem with the output has occurred
	 */
	public void writeToDisk(BufferedWriter file) throws IOException
	{
		file.write("\t<Card>\n");
		file.write("\t\t<question>" + this.question + "</question>\n");
		file.write("\t\t<answer>" + this.answer + "</answer>\n");
		file.write("\t\t<hint>" + this.hint + "</hint>\n");
		file.write("\t\t<info>" + this.additionalInfo + "</info>\n");
		file.write("\t\t<options>" + this.options.toString() + "</options>\n");
		file.write("\t</Card>\n");
	}
}
