/**
 * Deck.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.zip.DataFormatException;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;

/**
 * {@link Deck} represents the high-level intuition of a deck of flash cards.
 * The class stores a set of {@link Card}s and set of statistics based on past tests.
 * 
 * @author Jesse Yates, Greg Anderson, H. Parker Shelton
 */
public class Deck
{
	/** The location of this <code>Deck</code> on disk */
	private String diskLocation;

	/** The name of this <code>Deck</code> */
	private String name;

	/** The {@link Card} current selected */
	private int currentCard;

	/** The set of statistics for this <code>Deck</code>, compiled over time */
	private Collection<DeckStat> deckStat;

	/** The set of cards contained in this <code>Deck</code> */
	private List<Card> cards;

	/**
	 * Initialize a basic <code>Deck</code> using the name provided
	 * 
	 * @param name
	 *        A String representing the name of the <code>Deck</code>
	 */
	public Deck(String name)
	{
		this.name = name;
		// the card list should be an arraylist to optimize for arbitrary indexing (through the deck list GUI)
		this.cards = new ArrayList<Card>();
		this.deckStat = new ArrayList<DeckStat>();
	}

	/**
	 * Add the specified {@link Card} to the <code>Deck</code>
	 * 
	 * @param c
	 *        a {@link Card} to be added to the deck
	 */
	public boolean addCard(Card c)
	{
		return this.cards.add(c);
	}

	/**
	 * Add the list of cards to this deck
	 * 
	 * @param cards
	 *        A <code>List</code> of cards to add to this deck
	 * @return <code>true</code> if all cards were successfully added to the deck, <code>false</code> otherwise
	 */
	public boolean addCards(List<Card> cards)
	{
		this.cards.addAll(cards);
		return true;
	}

	/**
	 * Add the specified {@link Card} to the <code>Deck</code> at position <code>i</code>
	 * 
	 * @param i
	 *        An <code>int</code> representing the position at which to add the specified card
	 * @param c
	 *        The <code>Card</code> to be added
	 */
	public void addCardAtIndex(int i, Card c)
	{
		this.cards.add(i, c);
	}

	/**
	 * Adds a new testing statistic to the current set of statistics for the deck
	 * 
	 * @param d
	 *        {@link DeckStat} to be associated with the deck
	 * @return <code>true</code> if the {@link DeckStat} is inserted, <code>false</code> otherwise
	 */
	public boolean addDeckStat(DeckStat d)
	{
		return this.deckStat.add(d);
	}

	/**
	 * Removes all {@link Card}s from the <code>Deck</code>
	 * 
	 * @see java.util.Collection#clear()
	 */
	public void clearCards()
	{
		this.cards.clear();
	}

	/**
	 * Clears all of the statistics associated with this <code>Deck</code>
	 * 
	 * @see java.util.Collection#clear()
	 */
	public void clearDeckStat()
	{
		this.deckStat.clear();
	}

	/**
	 * Returns <code>true</code> if this <code>Deck</code> contains all specified {@link Card}s
	 * 
	 * @param cards
	 *        A <code>Collection<code> to be checked for containment in this collection
	 * @see java.util.Collection#containsAll(Collection)
	 */
	public boolean containsAllCards(Collection<Card> cards)
	{
		return this.cards.containsAll(cards);
	}

	/**
	 * Returns <code>true</code> if this <code>Deck</code> contains the specified {@link Card}
	 * 
	 * @param c
	 *        the {@link Card} whose presence in this <code>Deck</code> is to be tested
	 * @see java.util.Collection#contains(Object)
	 */
	public boolean containsCard(Card c)
	{
		return this.cards.contains(c);
	}

	/**
	 * Returns an iterator over this {@code Deck}'s statistics
	 * 
	 * @return an iterator over the collection of {@link DeckStat}s of this {@code Deck}
	 * @see java.util.Collection#iterator()
	 */
	public Iterator<DeckStat> deckStatisticsIterator()
	{
		return this.deckStat.iterator();
	}

	/**
	 * Returns the {@link Collection} of {@link Card}s in this <code>Deck</code>
	 * 
	 * @return the {@link Collection} of {@link Card}s in this <code>Deck</code>
	 */
	public List<Card> getCards()
	{
		return this.cards;
	}

	/**
	 * Returns the {@link Card} at the index of this deck
	 * 
	 * @param index
	 * @return The card at the index of this deck
	 */
	public Card getCard(int index)
	{
		return cards.get(index);
	}

	/**
	 * Returns a {@link Card} representing the currently selected card
	 * 
	 * @return a {@link Card} representing the currently selected card
	 */
	public Card getCurrentCard()
	{
		return this.cards.get(this.currentCard);
	}

	/**
	 * Returns a {@link Collection} of {@link DeckStat}s representing the cumulative statistics of this deck
	 * 
	 * @return a {@link Collection} of {@link DeckStat}s representing the cumulative statistics of this deck
	 */
	public Collection<DeckStat> getDeckStatistics()
	{
		return this.deckStat;
	}

	/**
	 * Returns a {@code String} representing the location of this {@code Deck} on disk
	 * 
	 * @return a {@code String} representing the location of this {@code Deck} on disk
	 */
	public String getDiskLocation()
	{
		return this.diskLocation;
	}

	/**
	 * Returns a {@code String} representing the name of this {@code Deck}
	 * 
	 * @return a {@code String} representing the name of this {@code Deck}
	 */
	public String getName()
	{
		return this.name;
	}

	/**
	 * Returns <code>true</code> if there are no statistics
	 * 
	 * @return <code>true</code> if there are no statistics
	 * @see java.util.Collection#isEmpty()
	 */
	public boolean isDeckStatEmpty()
	{
		return this.deckStat.isEmpty();
	}

	/**
	 * Returns the number of cards in this deck
	 * 
	 * @return the number of cards in this deck
	 */
	public int numCards()
	{
		return this.cards.size();
	}

	/**
	 * Returns the number of test statistics in this deck
	 * 
	 * @return the number of test statistics in this deck
	 * @see java.util.Collection#size()
	 */
	public int numDeckStatistics()
	{
		return this.deckStat.size();
	}

	/**
	 * Read a <code>Deck</code> from disk given an absolute file name
	 * 
	 * @param parent
	 *        A <code>QWidget</code> representing the parent <code>QWidget</code>
	 * @param filepath
	 *        A <code>String</code> representing the absolute path file name of the file
	 * @return a {@link Deck} created from the specified file
	 */
	public static Deck readFromDisk(QWidget parent, String filepath)
	{
		try
		{
			if(!new File(filepath).exists())
				throw new FileNotFoundException();

			BufferedReader file = new BufferedReader(new FileReader(filepath));

			if(!file.readLine().equals("<Deck>"))
				throw new DataFormatException();

			String line = file.readLine().trim();
			if(!line.contains("<name>"))
				throw new DataFormatException();
			String name = line.substring(line.indexOf("<name>") + 6, line.lastIndexOf("</name>"));

			Deck d = new Deck(name);

			line = file.readLine().trim();
			if(!line.equals("<DeckStat>") && !line.equals("<Card>") && !line.equals("</Deck>"))
				throw new DataFormatException();
			while(line.equals("<DeckStat>"))
			{
				d.addDeckStat(DeckStat.readFromDisk(file));
				line = file.readLine().trim();
			}

			if(!line.equals("<Card>") && !line.equals("</Deck>"))
				throw new DataFormatException();
			while(line.equals("<Card>"))
			{
				d.addCard(Card.readFromDisk(file));
				line = file.readLine().trim();
			}

			if(!line.equals("</Deck>"))
				throw new DataFormatException();
			file.close();

			// set the first card to the "current" card
			d.updateCurrentCard();
			d.setDiskLocation(filepath);

			return d;
		}
		catch(FileNotFoundException e)
		{
			e.printStackTrace();
			QMessageBox.critical(parent, "FileNotFoundException", "The selected file does not exist!");
		}
		catch(IOException e)
		{
			e.printStackTrace();
			QMessageBox.critical(parent, "IOException", "An I/O error has occurred!");
		}
		catch(DataFormatException e)
		{
			e.printStackTrace();
			QMessageBox.critical(parent, "DataFormatException", "The selected deck is improperly formatted!");
		}
		catch(ParseException e)
		{
			e.printStackTrace();
			QMessageBox.critical(parent, "ParseException", "The selected deck is improperly formatted!");
		}
		return null;
	}

	/**
	 * Removes the specified {@link Card} from the <code>Deck</code>, if it is present
	 * 
	 * @param c
	 *        the {@link Card} to be removed from the <code>Deck</code>, if it is present
	 * @see java.util.Collection#remove(Object)
	 */
	public boolean removeCard(Card c)
	{
		return this.cards.remove(c);
	}

	/**
	 * Gets the next <code>Card</code> in the deck
	 * 
	 * @return the next card in the deck after the current card
	 */
	public Card selectNextCard()
	{
		if(this.currentCard + 1 < this.cards.size())
			this.setCurrentCard(this.currentCard + 1);
		else
			this.setCurrentCard(0);

		return this.cards.get(this.currentCard);
	}

	/**
	 * Gets the previous <code>Card</code> in the deck
	 * 
	 * @return the previous card in the deck before the current card
	 */
	public Card selectPreviousCard()
	{
		if(this.currentCard > 0)
			this.setCurrentCard(this.currentCard - 1);
		else
			this.setCurrentCard(this.cards.size() - 1);

		return this.cards.get(this.currentCard);
	}

	/**
	 * Updates the currently selected {@code Card} to the specified {@link Card}
	 * 
	 * @param c
	 *        a {@link Card} representing the currently selected {@link Card}
	 */
	public void setCurrentCard(Card c)
	{
		int i = this.cards.lastIndexOf(c);
		if(i < 0)
			this.currentCard = 0;
		else
			this.currentCard = i;
	}

	/**
	 * Updates the currently selected <code>Card</code> to the <code>Card</code> at the specified index
	 * 
	 * @param i
	 *        an <code>int</code> representing the index of the currently selected {@link Card}
	 */
	public void setCurrentCard(int i)
	{
		this.currentCard = i;
	}

	/**
	 * Updates the location of this <code>Deck</code> on disk
	 * 
	 * @param d
	 *        a <code>String</code> representing the new location of this <code>Deck</code> on disk
	 */
	public void setDiskLocation(String d)
	{
		this.diskLocation = new String(d);
	}

	/**
	 * Updates the name of this <code>Deck</code> to the specified <code>String</code>
	 * 
	 * @param n
	 *        a <code>String</code> representing the new name of this <code>Deck</code>
	 */
	public void setName(String n)
	{
		this.name = n;
	}

	/**
	 * Randomizes the ordering of the cards in the card list, "shuffling" them
	 */
	public void shuffle()
	{
		if(cards.size() <= 1)
			return;

		Card current = this.cards.get(this.currentCard);

		ArrayList<Card> reordered = new ArrayList<Card>();
		Random rand = new Random();

		// Remove a uniformly distributed card from the current card list and add it to a temporary list
		while(this.cards.size() > 0)
			reordered.add(this.cards.remove(rand.nextInt(this.cards.size())));

		this.cards = reordered;
		this.currentCard = this.cards.indexOf(current);
	}

	/**
	 * Sets the current card to the first card of the deck
	 */
	public void updateCurrentCard()
	{
		if(this.cards != null && this.cards.size() > 0)
			this.currentCard = 0;
	}

	/**
	 * Export the contents of this <code>Deck</code> to disk
	 * 
	 * @param parent
	 *        A <code>QWidget</code> representing the parent <code>QWidget</code>
	 * @param filepath
	 *        A <code>String</code> representing the absolute path file name of the file
	 */
	public void writeToDisk(QWidget parent, String filepath)
	{
		try
		{
			System.out.println(filepath);
			if(!Preferences.getInstance().hasExportLocation())
				throw new IOException();

			File f = new File(filepath);
			if(!f.exists())
				f.createNewFile();

			BufferedWriter file = new BufferedWriter(new FileWriter(f));

			file.write("<Deck>\n");
			file.write("\t<name>" + this.name + "</name>\n");

			for(DeckStat d : this.deckStat)
				d.writeToDisk(file);
			for(Card c : this.cards)
				c.writeToDisk(file);

			file.write("</Deck>");
			file.close();
			this.diskLocation = new String(filepath);
		}
		catch(IOException e)
		{
			QMessageBox.critical(parent, "IOException", "An I/O error has occurred!");
			e.printStackTrace();
		}
	}
}
