/**
 * DeckStat.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.zip.DataFormatException;

/**
 * Contains statistics for a single test of a {@link Deck}
 * 
 * @author Jesse Yates
 */
public class DeckStat
{
	/** The number of questions answered in this test */
	private int numAnswered;
	/** The number of questions answered correctly in this test */
	private int numCorrect;

	// Date = new Date() sets it to the number of seconds since "epoch"
	/** The date and time the test began */
	private Date date;

	/** The duration of this test in seconds */
	private int duration;

	/**
	 * Initializes a <code>DeckStat</code> with the specified parameters
	 * 
	 * @param numA
	 *        an <code>int</code> representing the number of questions answered in a particular test
	 * @param numC
	 *        an <code>int</code> representing the number of questions answered correctly in a particular test
	 * @param d
	 *        an {@link Date} representing the date and time of a particular test
	 * @param duration
	 *        an <code>int</code> representing the duration in seconds of a particular test
	 */
	public DeckStat(int numA, int numC, Date d, int duration)
	{
		this.numAnswered = numA;
		this.numCorrect = numC;
		this.date = d;
		this.duration = duration;
	}

	/**
	 * Initializes an empty <code>DeckStat</code> with the current date and time
	 */
	public DeckStat(long currentTimeMillis)
	{
		this.date = new Date(currentTimeMillis);
		this.numAnswered = 0;
		this.numCorrect = 0;
		this.duration = 0;
	}

	/**
	 * Returns a <code>String</code> representing the date and time of this test
	 * 
	 * @return a <code>String</code> representing the date and time of this test
	 */
	public String getDate()
	{
		return date.toString();
	}

	/**
	 * Returns the test's duration in seconds
	 * 
	 * @return an <code>int</code> representing the duration of the test
	 */
	public int getDuration()
	{
		return this.duration;
	}

	/**
	 * Returns a <code>double</code> representing the percent accuracy on this test
	 * 
	 * @return a <code>double</code> representing the percent accuracy on this test
	 */

	public double getPercentCorrect()
	{
		return (numAnswered != 0) ? ((double) numCorrect) / numAnswered : 0;
	}

	/**
	 * Sets the duration of the test to the specified <code>int</code>
	 * 
	 * @param t
	 *        an <code>int</code> representing the length of time the test took
	 */
	public void setDuration(int t)
	{
		this.duration = t;
	}

	/**
	 * Sets the date of the test to the specified {@link Date}
	 * 
	 * @param d
	 *        a {@link Date} representing the date and time of the test
	 */
	public void setDate(Date d)
	{
		this.date = d;
	}

	/**
	 * Sets the date of the test to the specified <code>int</code>
	 * 
	 * @param d
	 *        a <code>int</code> representing the date and time of the test in seconds since Jan 1, 1970
	 */
	public void setDate(int d)
	{
		this.date = new Date(d);
	}

	/**
	 * Increments the overall number of questions answered and the number correctly answered if
	 * the answer was correct
	 * 
	 * @param correct
	 *        <code>true</code> if the question was answered correctly during the test, <code>false</code> otherwise
	 */
	public void incrementPercent(boolean correct)
	{
		this.numAnswered++;
		if(correct)
			this.numCorrect++;
	}

	/**
	 * Import a <code>DeckStat</code> from disk
	 * 
	 * @param file
	 *        A {@link BufferedReader} connected to a file on disk and positioned at the start of a
	 *        <code>DeckStat</code>
	 * @return a new <code>DeckStat</code> created from the file
	 * @throws IOException
	 *         if a problem with the input has occurred
	 * @throws DataFormatException
	 *         if data is found to be in the incorrect format
	 * @throws ParseException
	 *         is problem occurred when parsing the date
	 */
	public static DeckStat readFromDisk(BufferedReader file) throws IOException, DataFormatException, ParseException
	{
		String line = file.readLine().trim();

		if(!(line.contains("<date>") && line.contains("</date>")))
			throw new DataFormatException();
		String d = line.substring(line.indexOf("<date>") + 6, line.lastIndexOf("</date>"));
		DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT);
		Date date = df.parse(d);

		line = file.readLine().trim();
		if(!(line.contains("<numCorrect>") && line.contains("</numCorrect>")))
			throw new DataFormatException();
		int numC = Integer.parseInt(line.substring(line.indexOf("<numCorrect>") + 12, line.lastIndexOf("</numCorrect>")));

		line = file.readLine().trim();
		if(!(line.contains("<numAnswered>") && line.contains("</numAnswered>")))
			throw new DataFormatException();
		int numA = Integer.parseInt(line.substring(line.indexOf("<numAnswered>") + 13, line.lastIndexOf("</numAnswered>")));

		line = file.readLine().trim();
		if(!(line.contains("<duration>") && line.contains("</duration>")))
			throw new DataFormatException();
		int duration = Integer.parseInt(line.substring(line.indexOf("<duration>") + 10, line.lastIndexOf("</duration>")));

		if(!file.readLine().equals("\t</DeckStat>"))
			throw new DataFormatException();
		return new DeckStat(numC, numA, date, duration);
	}

	/**
	 * Export the contents of this <code>DeckStat</code> to disk
	 * 
	 * @param file
	 *        A {@link BufferedWriter} connected to a file on disk, positioned at the start of a <code>DeckStat</code>
	 * @throws IOException
	 *         if a problem during output has occurred
	 */
	public void writeToDisk(BufferedWriter file) throws IOException
	{
		file.write("\t<DeckStat>\n");
		file.write("\t\t<date>" + DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT).format(this.date) + "</date>\n");
		file.write("\t\t<numCorrect>" + numCorrect + "</numCorrect>\n");
		file.write("\t\t<numAnswered>" + numAnswered + "</numAnswered>\n");
		file.write("\t\t<duration>" + duration + "</duration>\n");
		file.write("\t</DeckStat>\n");
	}
}
