/**
 * DeckTreeModel.java
 * Group 7, Object Oriented Software Engineering
 * Oct 25, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.util.ArrayList;
import java.util.List;
import com.trolltech.qt.core.QMimeData;
import com.trolltech.qt.core.QModelIndex;
import com.trolltech.qt.core.QObject;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.core.Qt.ItemFlag;
import com.trolltech.qt.core.Qt.ItemFlags;
import com.trolltech.qt.gui.QTreeModel;

/**
 * The model for the deck tree view that provides the data
 * for the tree to populate
 * 
 * @author Pablo Lee
 */
public class DeckTreeModel extends QTreeModel
{
	/** The list of decks */
	private List<Deck> decks;
	
	/** The currently selected deck */
	private Deck currentDeck;

	/** Properties of the items in the tree */
	private static final ItemFlags defaultFlags = new ItemFlags(ItemFlag.ItemIsDragEnabled, ItemFlag.ItemIsDropEnabled, ItemFlag.ItemIsSelectable, ItemFlag.ItemIsEnabled);

	/**
	 * Initializes the deck tree
	 * 
	 * @param decks all of the {@link Deck}s
	 * @param currentDeck the <code>deck</code> we are currently on and wish to open
	 */
	public DeckTreeModel(List<Deck> decks, Deck currentDeck)
	{
		this.decks = decks;
		this.currentDeck = currentDeck;
	}

	/**
	 * Returns the default flags
	 * @param index
	 * @return the default flags in the form of the QT ItemFlags object
	 */
	public ItemFlags flags(QModelIndex index)
	{
		return defaultFlags;
	}

	/**
	 * Returns the child specified by <code>index</code>, in the given object
	 * if a deck is passed in a card at index is returned, else deck at index returned
	 * 
	 * @param item the deck
	 * @param index the index of the card we wish to return
	 * @return the card
	 */
	@Override
	public Object child(Object item, int index)
	{
		if(isDeck(item))
		{
			return ((Deck) item).getCard(index);
		}

		if(!isCard(item))
		{
			return decks.get(index);
		}
		return null;
	}

	/**
	 * Return the number of children of item, if a deck return num of cards,
	 * if item is not already a deck return the num of decks
	 * 
	 * @param item the object with children
	 */
	@Override
	public int childCount(Object item)
	{
		if(isDeck(item))
		{
			return ((Deck) item).getCards().size();
		}

		if(!isCard(item))
		{
			return decks.size();
		}
		return 0;
	}

	/**
	 * Returns the text of the object passed in, if a <code>deck</code> object the
	 * name of the <code>deck</code> is return, if a <code>card</code> object the question on the <code>card</code> is returned
	 * 
	 * @param item the object we want to return the text of
	 * @return the text of the object
	 */
	@Override
	public String text(Object item)
	{
		if(isDeck(item))
		{
			return ((Deck) item).getName();
		}
		else if(isCard(item))
		{
			return ((Card) item).getQuestion();
		}
		return null;
	}

	/**
	 * return the current <code>deck</code> the user is on
	 * 
	 * @return the current <code>deck</code>
	 */
	public Deck getCurrentDeck()
	{
		return this.currentDeck;
	}

	/**
	 * sets the current <code>deck</code> to the new deck passed in
	 * 
	 * @param deck the new <code>deck</code> 
	 */
	public void setCurrentDeck(Deck deck)
	{
		this.currentDeck = deck;
	}

	/**
	 * adds {@link Deck} to the set of decks
	 * 
	 * @param deck the new <code>deck</code> to be added
	 */
	public void addDeck(Deck deck)
	{
		if(deck.getCards().size() > 0)
		{
			deck.setCurrentCard(deck.getCard(0));
			this.currentDeck = deck;
		}
		decks.add(deck);
		int newPos = decks.indexOf(deck);

		// Update the tree view
		childrenInserted(null, newPos, newPos);
	}

	/**
	 * returns the list of all decks
	 * 
	 * @return a list of decks
	 */
	public List<Deck> getDecks()
	{
		return decks;
	}

	/**
	 * Helper method to check if an object is a <code>Deck</code>
	 * 
	 * @return true is item is a Deck, false otherwise
	 */
	public Boolean isDeck(Object item)
	{
		if(item instanceof Deck)
		{
			return true;
		}
		return false;
	}

	/**
	 * Helper method to check if an object's a {@link Card}
	 * 
	 * @return true is item is a <code>Card</code>, false otherwise
	 */
	public Boolean isCard(Object item)
	{
		if(item instanceof Card)
		{
			return true;
		}
		return false;
	}

	/**
	 * returns the list of string types
	 * 
	 * @return the list of types
	 */
	public List<String> mimeTypes()
	{
		List<String> types = new ArrayList<String>();
		types.add("text/plain");
		return types;
	}

	/**
	 * When setting the mimeData for drag and drop events (mimeData's
	 * how info gets passed on what's been dragged or dropped), use
	 * the new custom mimeData class that also keeps track of what item
	 * is being dragged by overriding this method that returns the mimeData
	 * 
	 * @return the data found in the list or null if not data in list
	 */
	public QMimeData mimeData(List<QModelIndex> list)
	{
		if(list.size() > 0)
		{
			Object item = indexToValue(list.get(0));
			TreeItemMimeData mimeData = new TreeItemMimeData(this);
			mimeData.item = item;
			mimeData.setText(item.toString());
			return mimeData;
		}
		return null;
	}

	/**
	 * The method called after something's been dropped in an item
	 * where objects can be dropped on
	 */
	public boolean dropMimeData(QMimeData data, Qt.DropAction action, int row, int col, QModelIndex parentIndex)
	{
		Object receivingItem = indexToValue(parentIndex);

		if(data instanceof TreeItemMimeData)
		{
			Object draggedItem = ((TreeItemMimeData) data).item;

			// If you're moving a card to a new deck
			if(isCard(draggedItem) && isDeck(receivingItem))
			{
				Card movingCard = (Card) draggedItem;

				Deck oldDeck = (Deck) indexToValue(parent(valueToIndex(draggedItem)));
				Deck newDeck = (Deck) receivingItem;

				int oldPos = oldDeck.getCards().indexOf(movingCard);
				oldDeck.removeCard(movingCard);
				childrenRemoved(valueToIndex(oldDeck), oldPos, oldPos);

				int newPos = row;
				if(row < 0 || row > newDeck.getCards().size())
				{
					newPos = newDeck.getCards().size();
				}
				newDeck.addCardAtIndex(newPos, movingCard);
				childrenInserted(valueToIndex(newDeck), newPos, newPos);
			}

			// If you're moving a deck to a new position
			if(isDeck(draggedItem) && !isDeck(receivingItem) && !isCard(receivingItem))
			{
				Deck movingDeck = (Deck) draggedItem;

				int oldPos = decks.indexOf(movingDeck);
				decks.remove(oldPos);
				childrenRemoved(null, oldPos, oldPos);

				int newPos = row;
				if(row < 0 || row > decks.size())
				{
					newPos = decks.size();
				}
				decks.add(newPos, movingDeck);
				childrenInserted(null, newPos, newPos);
			}

			// If you're merging two decks
			if(isDeck(draggedItem) && isDeck(receivingItem))
			{
				Deck movingDeck = (Deck) draggedItem;

				int oldPos = decks.indexOf(movingDeck);
				List<Card> cards = movingDeck.getCards();
				decks.remove(oldPos);
				childrenRemoved(null, oldPos, oldPos);

				int oldNumCards = ((Deck) receivingItem).numCards();

				((Deck) receivingItem).addCards(cards);
				reloadDeck((Deck) receivingItem, oldNumCards);
			}
		}

		return false;
	}

	/**
	 * Reloads a deck in the tree view by removing its cards and
	 * adding them back again
	 * 
	 * @param deck the <code>deck</code> to be updated
	 * @param oldNumCards the previous number of cards the <code>deck</code> had
	 */
	public void reloadDeck(Deck deck, int oldNumCards)
	{
		QModelIndex deckIndex = valueToIndex(deck);
		int firstPos = 0;
		int lastPos = childCount(deck) - 1;

		childrenRemoved(deckIndex, firstPos, oldNumCards - 1);
		childrenInserted(deckIndex, firstPos, lastPos);
	}
}

/**
 * Subclass of QMimeData to also keep track of the object
 * being dragged
 * 
 * @author Pablo Lee
 *
 */
class TreeItemMimeData extends QMimeData
{
	public TreeItemMimeData(QObject object)
	{
	}

	/** The item being dragged */
	public Object item;

	/**
	 * Defines how the Mime data should be printed out when it is an instance of a tree item
	 */
	public String toString()
	{
		return "NodeRefMimeData(" + item.toString() + ")";
	}
}
