/**
 * EditCardGUI.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.util.HashMap;
import java.util.Map.Entry;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMainWindow;
import com.trolltech.qt.gui.QMenu;
import com.trolltech.qt.gui.QMenuBar;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

/**
 * GUI responsible for the view and controller for the underlying {@link EditCardWindow} model
 * 
 * @author Jesse Yates
 */
public class EditCardGUI extends QMainWindow
{
	/** The underlying {@link EditCardWindow} model */
	private EditCardWindow editWindow;

	/** The <code>Card</code> to be edited */
	@SuppressWarnings("unused")
	private Card card;

	/** The Question in displayable format */
	private QWidget question;

	/** The answer in displayable format */
	private QWidget answer;

	/** The hint in displayable format */
	private QTextEdit hint;

	/** The hint in displayable format */
	private QTextEdit addInfo;

	/** A {@link HashMap} representing the preferences of the <code>Card</code> */
	private HashMap<String, Boolean> pref;

	/** A QMenu representing the file menu */
	private QMenu fileMenu;

	/** A QMenu representing the edit menu */
	@SuppressWarnings("unused")
	private QMenu editMenu;

	/** A QMenu representing the help menu */
	private QMenu helpMenu;

	/** A QAction that displays the about dialog */
	private QAction aboutAct;

	/** A QAction that saves the current <code>Card</code> */
	private QAction saveAct;

	/** A QAction that closes the current dialog */
	private QAction closeAct;

	/**
	 * Constructor for the GUI
	 * 
	 * @param parent
	 *        a {@link MainWindow} representing the main window of the application
	 * @param editWindow
	 *        a {@link EditCardWindow} representing the underlying model for the GUI
	 */
	public EditCardGUI(MainWindow parent, EditCardWindow editWindow)
	{
		super(parent);
		this.setMenuBar(new QMenuBar());
		this.editWindow = editWindow;
		this.pref = new HashMap<String, Boolean>();

		// setup the window for display
		this.addToolBar(Qt.ToolBarArea.LeftToolBarArea, editWindow.getToolbar());
		this.setUpCard();

		QWidget editingArea = new QWidget();
		QGridLayout layout = new QGridLayout(editingArea);
		editingArea.setLayout(layout);

		layout.setHorizontalSpacing(3);
		layout.setVerticalSpacing(3);
		layout.setMargin(5);
		QVBoxLayout leftSide = new QVBoxLayout();
		QVBoxLayout rightSide = new QVBoxLayout();
		layout.addLayout(leftSide, 0, 0);
		layout.addLayout(rightSide, 0, 1);

		QVBoxLayout settingsLayout = new QVBoxLayout();
		settingsLayout.addWidget(new QLabel("Settings"));
		for(Entry<String, Boolean> i : this.pref.entrySet())
		{
			PreferenceCheckBox box = new PreferenceCheckBox(i.getKey());
			if(i.getValue().booleanValue())
				box.setCheckState(Qt.CheckState.Checked);
			box.stateChange.connect(this, "updateSetting(String, boolean)");
			settingsLayout.addWidget(box);
		}

		layout.setColumnMinimumWidth(0, 100);
		layout.setColumnMinimumWidth(1, 50);
		layout.setRowMinimumHeight(0, 200);

		leftSide.addWidget(new QLabel("Question"), 0, Qt.AlignmentFlag.AlignCenter);
		leftSide.addWidget(this.question);

		leftSide.addWidget(new QLabel("Answer"), 0, Qt.AlignmentFlag.AlignCenter);
		leftSide.addWidget(this.answer);

		rightSide.setSpacing(10);
		rightSide.addWidget(new QLabel("Hint"), 0, Qt.AlignmentFlag.AlignCenter);
		rightSide.addWidget(this.hint);
		rightSide.addWidget(new QLabel("Additional Information"), 0, Qt.AlignmentFlag.AlignCenter);
		rightSide.addWidget(this.addInfo);
		rightSide.addLayout(settingsLayout);
		rightSide.setAlignment(settingsLayout, Qt.AlignmentFlag.AlignCenter);

		this.setCentralWidget(editingArea);

		setupActions();
		setupMenu();
	}

	/**
	 * Display information about editing a <code>Card</code>
	 */
	public void about()
	{
		QMessageBox.information(this, tr("Editing a Card"), tr("Different toolbars allow for different editing. \nSimply put, input the question, answer, hint and additional information as you see fit, and then select the options for the card. Finally save the card."));
	}

	/**
	 * If the window attempts to close, determine if the card should be saved
	 * 
	 * @see com.trolltech.qt.gui.QWidget#closeEvent(com.trolltech.qt.gui.QCloseEvent)
	 */
	public void closeEvent(QCloseEvent event)
	{
		if(maybeSave())
			event.accept();
		else
			event.ignore();
	}

	/**
	 * Prompt the user to determine if the <code>Card</code> should be saved
	 * 
	 * @return <code>true</code> if the <code>Card</code> should be saved
	 */
	private boolean displaySavePrompt()
	{
		QMessageBox.StandardButton ret = QMessageBox.warning(this, tr("Edit Card"), "The card has been modified.\n" + "Save your changes?", new QMessageBox.StandardButtons(QMessageBox.StandardButton.Ok, QMessageBox.StandardButton.No, QMessageBox.StandardButton.Cancel));
		if(ret == QMessageBox.StandardButton.Ok)
			saveCard();
		else if(ret == QMessageBox.StandardButton.Cancel)
			return false;

		return true;
	}

	/**
	 * Returns the {@link EditCardWindow} model this GUI is associated with
	 * 
	 * @return the {@link EditCardWindow} model this GUI is associated with
	 */
	public EditCardWindow getEditWindow()
	{
		return this.editWindow;
	}

	/**
	 * Check to see the <code>card</code> should be saved
	 * 
	 * @return True if the <code>card</code> should be saved, false otherwise
	 */
	private boolean maybeSave()
	{
		if(this.editWindow.getCard().checkAnswer(this.answer.toString()))
			if(this.editWindow.getCard().getQuestion().equals(this.question.toString()))
				if(this.editWindow.getCard().getHint().equals(this.hint.toPlainText()))
					if(this.editWindow.getCard().getAdditionalInfo().equals(this.addInfo.toPlainText()))
						if(this.pref.equals(this.editWindow.getCard().options))
							return true;
		return displaySavePrompt();
	}

	/**
	 * Save all the current values to the <code>Card</code>
	 */
	private void saveCard()
	{
		this.editWindow.save(this.question.toString(), this.answer.toString(), this.hint.toPlainText(), this.addInfo.toPlainText(), this.pref);
		// XXX - HPS: Correct behavior is to save the card modifications and user manually saves deck as well
		// MainWindow w = (MainWindow) this.parent();
		// w.saveDeck();
	}

	/**
	 * Initialize the actions in the menus
	 */
	private void setupActions()
	{
		this.saveAct = new QAction(tr("&Save"), this);
		this.saveAct.setShortcut(tr("Ctrl+S"));
		this.saveAct.setStatusTip(tr("Save the current card"));
		this.saveAct.triggered.connect(this, "saveCard()");

		this.closeAct = new QAction(tr("&Exit"), this);
		this.closeAct.setShortcut(tr("Ctrl+X"));
		this.closeAct.triggered.connect(this, "close()");

		this.aboutAct = new QAction(tr("&About"), this);
		this.aboutAct.setShortcut(tr("Ctrl+A"));
		this.aboutAct.triggered.connect(this, "about()");
	}

	/**
	 * Get the information from the current {@link Card}
	 */
	private void setUpCard()
	{
		this.hint = new QTextEdit(this.editWindow.getCard().getHint());
		this.hint.setMinimumSize(20, 20);

		this.addInfo = new QTextEdit(this.editWindow.getCard().getAdditionalInfo());
		this.addInfo.setMinimumSize(20, 20);

		this.answer = this.editWindow.getToolbar().getFaceCanvas(Face.BACK, this.editWindow.getCard().getAnswer());

		this.question = this.editWindow.getToolbar().getFaceCanvas(Face.FRONT, this.editWindow.getCard().getQuestion());

		this.pref = new HashMap<String, Boolean>();
		for(Entry<String, Boolean> j : this.editWindow.getCard().options.entrySet())
			this.pref.put(new String(j.getKey()), new Boolean(j.getValue()));
	}

	/**
	 * Initialize the menus
	 */
	private void setupMenu()
	{
		this.fileMenu = menuBar().addMenu(tr("&File"));
		this.fileMenu.addAction(this.saveAct);
		this.fileMenu.addSeparator();
		this.fileMenu.addAction(this.closeAct);

		this.editMenu = menuBar().addMenu(tr("&Edit"));
		/*
		 * editMenu.addAction(cutAct);
		 * editMenu.addAction(copyAct);
		 * editMenu.addAction(pasteAct);
		 */
		menuBar().addSeparator();

		this.helpMenu = menuBar().addMenu(tr("&Help"));
		this.helpMenu.addAction(this.aboutAct);
	}

	/**
	 * Update the stored setting
	 * 
	 * @param k
	 *        The setting name
	 * @param v
	 *        The value to store
	 */
	public void updateSetting(String k, boolean v)
	{
		this.pref.put(k, new Boolean(v));
	}

}
