/**
 * MainWindow.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import com.trolltech.qt.core.QPoint;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QDockWidget;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QIcon;
import com.trolltech.qt.gui.QMainWindow;
import com.trolltech.qt.gui.QMenu;
import com.trolltech.qt.gui.QMenuBar;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QToolBar;
import com.trolltech.qt.gui.QFileDialog.Filter;

/**
 * Main window that calls the other subunits for the program.<br>
 * Some of these methods are at their initial stages and will further
 * extend their functionality in future iterations
 * 
 * @author H. Parker Shelton, Jesse Yates, Pablo Lee, Greg Anderson
 */
public class MainWindow extends QMainWindow
{
	/** The currently selected deck */
	// TODO - Needs to be remade to private after all the temp deck generation code has been removed
	public Deck currentDeck;

	/** List of all the decks available for selection */
	private List<Deck> decks;

	/** Tree widget for the deck of cards */
	private DeckTreeGUI treeView;

	/** Tree widget model */
	private DeckTreeModel treeModel;

	/** Dock widget to hold the tree widget */
	private QDockWidget dockTreeWidget;

	/** The current toolbar used for editing */
	private ToolBar currentToolbar;

	/** The toolbar used for file menu options */
	private QToolBar fileToolbar;

	/** The toolbar used for deck options */
	private QToolBar deckToolbar;

	/** QAction to create a new deck */
	private QAction newDeckAction;

	/** QAction to load a deck */
	private QAction loadDeckAction;

	/** QActions for saving decks */
	private QAction saveDeckAction;
	private QAction saveDeckAsAction;

	/** QAction for shuffling a deck */
	private QAction shuffleAction;

	/** QAction to display the help prompt */
	private QAction aboutELevelAction;

	/** QAction to display the preference prompt */
	private QAction setPreferencesAction;

	/** QAction to close the program */
	private QAction exitAction;

	/** Card QActions */
	private QAction newCardAction;
	private QAction editCardAction;
	private QAction testDeckAction;

	/** The central card area of the GUI */
	private CardAreaGUI cardArea;

	/**
	 * Initializes a new MainWindow
	 */
	public MainWindow()
	{
		decks = new ArrayList<Deck>();

		initializeActions();
		initializeMenuBar();
		initializeToolBars();
	}

	/**
	 * Initializes the QActions required for MainWindow
	 */
	private void initializeActions()
	{
		this.newDeckAction = new QAction(new QIcon("images/newDeck.png"), tr("New"), this);
		this.newDeckAction.setStatusTip(tr("Create a new Deck"));
		this.newDeckAction.triggered.connect(this, "newDeck()");

		this.loadDeckAction = new QAction(new QIcon("images/open.png"), tr("Open"), this);
		this.loadDeckAction.setStatusTip(tr("Open Selected Deck"));
		this.loadDeckAction.setShortcut(tr("Ctrl+O"));
		this.loadDeckAction.triggered.connect(this, "openDeck()");

		this.saveDeckAction = new QAction(new QIcon("images/save.png"), tr("Save"), this);
		this.saveDeckAction.setStatusTip(tr("Save Selected Deck"));
		this.saveDeckAction.setShortcut(tr("Ctrl+S"));
		this.saveDeckAction.triggered.connect(this, "saveDeck()");

		this.saveDeckAsAction = new QAction(new QIcon("images/saveAs.png"), tr("Save As"), this);
		this.saveDeckAsAction.setStatusTip(tr("Save Selected Deck As"));
		this.saveDeckAsAction.setShortcut(tr("Ctrl+Shift+S"));
		this.saveDeckAsAction.triggered.connect(this, "saveDeckAs()");

		this.shuffleAction = new QAction(new QIcon("images/shuffle.png"), tr("Shuffle"), this);
		this.shuffleAction.setStatusTip(tr("Shuffle Current Deck"));
		this.shuffleAction.triggered.connect(this, "shuffleCurrentDeck()");

		this.aboutELevelAction = new QAction(tr("About E-Level"), this);
		this.aboutELevelAction.setStatusTip(tr("About E-Level"));
		this.aboutELevelAction.triggered.connect(this, "aboutELevel()");

		this.setPreferencesAction = new QAction(tr("Preferences"), this);
		this.setPreferencesAction.setStatusTip(tr("Change Preferences"));
		this.setPreferencesAction.triggered.connect(this, "setPreferences()");

		this.newCardAction = new QAction(new QIcon("images/newCard.png"), tr("New Card"), this);
		this.newCardAction.setStatusTip(tr("Create a new Card"));
		this.newCardAction.triggered.connect(this, "newCard()");

		this.editCardAction = new QAction(new QIcon("images/editCard.png"), tr("Edit Card"), this);
		this.editCardAction.setStatusTip(tr("Edit this Card"));
		this.editCardAction.triggered.connect(this, "editCard()");

		this.testDeckAction = new QAction(new QIcon("images/test.png"), tr("Test"), this);
		this.testDeckAction.setStatusTip(tr("Test this deck"));
		this.testDeckAction.triggered.connect(this, "testCard()");

		this.exitAction = new QAction(tr("Exit"), this);
		this.exitAction.setStatusTip(tr("Exit E-Level"));
		this.exitAction.setShortcut(tr("Ctrl+W"));
		this.exitAction.triggered.connect(this, "close()");
	}

	/**
	 * Initializes the menu bar for MainWindow
	 */
	private void initializeMenuBar()
	{
		QMenuBar menuBar = new QMenuBar();
		setMenuBar(menuBar);

		QMenu fileMenu = menuBar.addMenu(tr("&File"));
		fileMenu.addAction(loadDeckAction);
		fileMenu.addAction(saveDeckAction);
		fileMenu.addAction(saveDeckAsAction);
		fileMenu.addSeparator();
		fileMenu.addAction(exitAction);

		QMenu deckMenu = menuBar.addMenu(tr("&Deck"));
		deckMenu.addAction(testDeckAction);
		deckMenu.addAction(newDeckAction);
		deckMenu.addAction(shuffleAction);

		QMenu cardMenu = deckMenu.addMenu(tr("&Card"));
		cardMenu.addAction(newCardAction);
		cardMenu.addAction(editCardAction);

		QMenu helpMenu = menuBar.addMenu(tr("&Help"));
		helpMenu.addAction(setPreferencesAction);
		helpMenu.addAction(aboutELevelAction);
	}

	/**
	 * Initializes the basic properties of each the toolbars used
	 */
	private void initializeToolBars()
	{
		this.currentToolbar = new DefaultToolBar();

		this.fileToolbar = addToolBar(tr("File"));
		this.fileToolbar.addAction(newDeckAction);
		this.fileToolbar.addAction(loadDeckAction);
		this.fileToolbar.addAction(saveDeckAction);
		this.fileToolbar.addAction(saveDeckAsAction);

		this.deckToolbar = addToolBar(tr("Deck"));
		this.deckToolbar.addAction(newCardAction);
		this.deckToolbar.addAction(editCardAction);
		this.deckToolbar.addAction(testDeckAction);
	}

	/*
	 * So there is a treeView and a tree model. Now, I thought of having this MainWindow
	 * class keep track of the list of decks / current deck, but then I'd have to make
	 * sure that synchronizes with that of the tree model, so instead I just made sure
	 * the only copy is in the model. Any changes in the tree model will cause the
	 * card area to update, and changes in the tree model are automatically reflected
	 * in the tree view.
	 */
	/**
	 * Sets up the basic tree view of the {@link Deck}s using the {@link DeckTreeModel}
	 */
	public void setupDeckTree()
	{
		// Create tree view
		treeModel = new DeckTreeModel(decks, currentDeck);
		treeView = new DeckTreeGUI(treeModel);
		treeView.setModel(treeModel);
		treeView.setCurrentIndex(treeModel.valueToIndex(treeModel.getCurrentDeck()));
		treeView.setMinimumWidth(200);

		dockTreeWidget = new QDockWidget(tr("Decks"), this);
		dockTreeWidget.setFeatures(QDockWidget.DockWidgetFeature.NoDockWidgetFeatures);
		dockTreeWidget.setAllowedAreas(Qt.DockWidgetArea.LeftDockWidgetArea);
		dockTreeWidget.setWidget(treeView);
		this.addDockWidget(Qt.DockWidgetArea.LeftDockWidgetArea, dockTreeWidget);
	}

	/**
	 * Sets up the tree view of the {@link Card}s under the currently selected Deck using {@link CardAreaGUI}
	 */
	public void setupCardArea()
	{
		Deck currentDeck = treeModel.getCurrentDeck();
		currentDeck.setCurrentCard(currentDeck.getCard(0));

		this.cardArea = new CardAreaGUI(currentDeck.getCurrentCard(), this);
		setCentralWidget(cardArea);
		treeView.selectedCardChanged.connect(this.cardArea, "displayCard(Card)");
	}

	/**
	 * Temporary method to generate sample decks
	 * TODO remove this method
	 */
	public void generateDeckandCards()
	{
		System.out.println("Creating new decks at " + Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION"));
		this.decks.add(new Deck("first"));
		this.decks.add(new Deck("second"));
		this.decks.get(0).addCard(new Card("Question1", "Answer1", "Hint1", "Info1"));
		this.decks.get(0).addCard(new Card("OtherQuestion1", "Answer1", "Hint1", "MoreInfo1"));
		this.decks.get(0).addCard(new Card("OtherOtherQuestion1", "Answer1", "Hint1", "MoreInfo1"));
		this.decks.get(0).addCard(new Card("FinalQuestion1", "Answer1", "Hint1", "MoreInfo1"));
		// this.decks.get(0).shuffle();
		this.decks.get(1).addCard(new Card("Question2", "Answer2", "Hint2", "Info2"));
		this.decks.get(0).setCurrentCard(0);
		this.decks.get(1).setCurrentCard(0);
		this.decks.get(0).addDeckStat(new DeckStat(10, 15, new Date(100), 100));
		this.decks.get(0).addDeckStat(new DeckStat(10, 20, new Date(50000), 100));

		for(int i = 0; i < this.decks.size(); i++)
			this.decks.get(i).writeToDisk(this, Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION") + String.valueOf(i + 1) + ".elvl");
		this.decks.clear();

		System.out.println("Opening 1.elvl by default!");
		this.decks.add(Deck.readFromDisk(this, Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION") + "1.elvl"));
		this.decks.add(Deck.readFromDisk(this, Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION") + "2.elvl"));
		this.currentDeck = this.decks.get(0);
	}

	/**
	 * Displays an information box about the program
	 */
	@SuppressWarnings("unused")
	private void aboutELevel()
	{
		QMessageBox.about(this, tr("About E-Level"), tr("<b>Iteration 3</b><br><br>" + "<b>Written by:</b><br>H. Parker Shelton - web \"development\", resident Qt \"expert\"<br>" + "Jesse Yates - the wild card<br>" + "Greg Anderson - algorithmic and concept design<br>" + "Pablo Lee - misc.<br>" + "Elaine Zapata - (GUI and group) beautification, model and code review<br>"));
	}

	/**
	 * Creates a new deck
	 */
	@SuppressWarnings("unused")
	private void newDeck()
	{
		System.out.println("Creating new deck");
		// Extension Example
		DeckDialog open = new DeckDialog(this);
		open.closing.connect(treeModel, "addDeck(Deck)");
		open.show();
	}

	/**
	 * Opens <code>Deck</code> from disk, prompting the user to select the deck on disk
	 */
	@SuppressWarnings("unused")
	private void openDeck()
	{
		String filepath = QFileDialog.getOpenFileName(this, tr("Open Deck"), (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION"), new Filter("*.elvl"));
		if(filepath == null || filepath.equals(""))
			return;
		Deck deck = Deck.readFromDisk(this, filepath);
		if(!treeModel.getDecks().contains(deck))
			treeModel.addDeck(deck);
	}

	/**
	 * Save the <code>deck</code> based on the deck's current location on disk
	 */
	@SuppressWarnings("unused")
	private void saveDeck() throws IOException
	{
		String filepath = treeModel.getCurrentDeck().getDiskLocation();
		if(filepath == null || filepath.equals(""))
			saveDeckAs();
		else
			treeModel.getCurrentDeck().writeToDisk(this, filepath);
	}

	/**
	 * Save the <code>Deck</code> by prompting the user for a location on disk
	 */
	private void saveDeckAs()
	{
		String filepath = QFileDialog.getSaveFileName(this, tr("Save Deck"), (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION"), new Filter("*.elvl"));
		if(filepath == null || filepath.equals(""))
			return;
		treeModel.getCurrentDeck().writeToDisk(this, filepath);
	}

	/**
	 * Calls the method to shuffle the <code>Card</code>s in the currently selected <code>Deck</code>
	 */
	@SuppressWarnings("unused")
	private void shuffleCurrentDeck()
	{
		treeModel.getCurrentDeck().shuffle();
		treeModel.reloadDeck(treeModel.getCurrentDeck(), treeModel.getCurrentDeck().numCards());
	}

	/**
	 * Display the settings that can be changed
	 */
	@SuppressWarnings("unused")
	private void setPreferences()
	{
		PreferenceWindow preferences = new PreferenceWindow(this);
		preferences.exec();
	}

	/**
	 * Displays an {@link EditCardGUI} populated with a blank card
	 */
	@SuppressWarnings("unused")
	private void newCard()
	{
		this.currentDeck.setCurrentCard(new Card());
		this.editCard();
	}

	/**
	 * Displays an {@link EditCardGUI} populated with the currently selected card
	 */
	private void editCard()
	{
		if(treeModel.getCurrentDeck() == null)
			this.currentDeck = new Deck("Temporary");
		// EditCardWindow edit = new EditCardWindow(this.currentDeck.getCurrentCard(), currentToolbar);
		EditCardWindow edit = new EditCardWindow(treeModel.getCurrentDeck(), currentToolbar);
		EditCardGUI editGUI = new EditCardGUI(this, edit);// , flags);
		editGUI.setWindowIcon(this.windowIcon());
		editGUI.setWindowTitle(tr("Edit Card"));
		QPoint alignment = this.geometry().topLeft();
		editGUI.setGeometry(alignment.x(), alignment.y(), 800, 400);
		editGUI.show();// FullScreen();
	}

	/**
	 * Displays the {@link TestWindowGUI}
	 */
	@SuppressWarnings("unused")
	private void testCard()
	{
		TestWindow test = new TestWindow(treeModel.getCurrentDeck());
		TestWindowGUI gui = new TestWindowGUI(test, this);
		gui.setWindowTitle("Testing");
		gui.show();// FullScreen();
	}

	/**
	 * Sets the previous <code>Card</code> in the <code>Deck</code> to the current card and displays it. Wraps to the
	 * last card in the deck.
	 */
	@SuppressWarnings("unused")
	private void previousCard()
	{
		treeModel.getCurrentDeck().selectPreviousCard();
		treeView.currentChanged(null, null);
	}

	/**
	 * Sets the next <code>Card</code> in the <code>Deck</code> to the current card and displays it. Wraps to the first
	 * card in the deck.
	 */
	@SuppressWarnings("unused")
	private void nextCard()
	{
		treeModel.getCurrentDeck().selectNextCard();
		treeView.currentChanged(null, null);
	}
}
