/**
 * PreferenceWindow.java
 * Group 7, Object Oriented Software Engineering
 * October 19, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QListWidget;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QStackedLayout;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

/**
 * Creates a Window in which the current program preference can be
 * changed and viewed
 * 
 * @author Jesse Yates, H. Parker Shelton
 */
public class PreferenceWindow extends QDialog
{
	/** The preferences and their input fields, stored to be written to disk later */
	Map<String, QLineEdit> textField;

	/** QWidgets for the different preference panes */
	QWidget preferences;
	QWidget testing;

	/** The QLayout for the window */
	QStackedLayout stackedLayout;

	/**
	 * Initializes a <code>PreferenceWindow</code> with the given parent
	 * 
	 * @param parent
	 *        A <code>QWidget</code> representing the parent window
	 */
	public PreferenceWindow(QWidget parent)
	{
		super(parent);

		QListWidget listWidget = new QListWidget();
		listWidget.addItem(tr("Program Settings"));
		listWidget.addItem(tr("Testing Settings"));
		listWidget.setFixedWidth(120);

		// QPushButton applyButton = new QPushButton(tr("Apply"));
		QPushButton closeButton = new QPushButton(tr("Close"));
		QPushButton revertButton = new QPushButton(tr("Restore to Defaults"));

		QHBoxLayout buttonsLayout = new QHBoxLayout();
		buttonsLayout.addWidget(revertButton);
		buttonsLayout.addStretch(1);
		// buttonsLayout.addWidget(applyButton);
		buttonsLayout.addWidget(closeButton);

		revertButton.pressed.connect(this, "restoreDefaults()");
		closeButton.pressed.connect(this, "close()");
		// closeButton.pressed.connect(this, "restorePrevious()");
		// applyButton.pressed.connect(this, "savePreferences()");

		this.stackedLayout = new QStackedLayout();
		listWidget.currentRowChanged.connect(this.stackedLayout, "setCurrentIndex(int)");
		listWidget.setCurrentRow(0);

		this.preferences = createProgramPreferences();
		this.testing = createTestingPreferences();
		this.stackedLayout.addWidget(this.preferences);
		this.stackedLayout.addWidget(this.testing);

		QHBoxLayout horizontalLayout = new QHBoxLayout();
		horizontalLayout.addWidget(listWidget);
		horizontalLayout.addLayout(this.stackedLayout);

		QVBoxLayout windowLayout = new QVBoxLayout();
		windowLayout.addLayout(horizontalLayout);
		windowLayout.addLayout(buttonsLayout);
		this.setLayout(windowLayout);

		setWindowTitle(tr("Preferences"));
	}

	/**
	 * Displays a dialog to change the default export directory
	 */
	public void changeExportLocation()
	{
		Preferences.getInstance().changeExportLocation(this, "Please select a new directory for E-Level decks:");
	}

	/**
	 * Writes out the preferences when the window is closed
	 */
	public void closeEvent(QCloseEvent event)
	{
		Preferences.getInstance().writeSettings();
	}

	/**
	 * Initializes a widget containing the preferences for the program overall
	 */
	private QWidget createProgramPreferences()
	{
		this.textField = new HashMap<String, QLineEdit>();

		QGridLayout layout = new QGridLayout();
		layout.addWidget(new QLabel("<b>Program Settings</b>"), 0, 0, 1, 2, Qt.AlignmentFlag.AlignCenter, Qt.AlignmentFlag.AlignTop);
		int j = 1;
		for(Entry<String, String> i : Preferences.getInstance().getEntrySet())
			if(i.getValue() != null)
				if(i.getValue().equals("true") || i.getValue().equals("false"))
				{
					PreferenceCheckBox box = new PreferenceCheckBox(i.getKey());
					if(i.getValue().equals("true"))
						box.setCheckState(Qt.CheckState.Checked);
					box.stateChange.connect(this, "updatePreference(String, boolean)");
					layout.addWidget(box, j++, 0, 1, 2, Qt.AlignmentFlag.AlignTop);
				}
				else if(i.getKey().equals("DEFAULT_EXPORT_LOCATION"))
				{
					String loc = Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION");
					layout.addWidget(new QLabel(loc.substring(0, 3) + "..." + loc.substring(loc.length() - 15)), j, 0, Qt.AlignmentFlag.AlignCenter);
					QPushButton b = new QPushButton(tr("Change Location"), this);
					b.clicked.connect(this, "changeExportLocation()");
					layout.addWidget(b, j++, 1, Qt.AlignmentFlag.AlignCenter);
				}
				else
				{
					QLabel label = new QLabel(i.getKey());
					layout.addWidget(label, j, 0, 1, 1);
					QLineEdit edit = new QLineEdit(i.getValue());
					this.textField.put(i.getKey(), edit);
					layout.addWidget(edit, j++, 1, 1, 1);
				}

		QWidget programPage = new QWidget();
		programPage.setLayout(layout);
		return programPage;
	}

	/**
	 * Initializes a widget containing the preferences for testing
	 */
	private QWidget createTestingPreferences()
	{
		QGridLayout layout = new QGridLayout();
		layout.addWidget(new QLabel("<b>Test Settings</b>"), 0, 0, 1, 2, Qt.AlignmentFlag.AlignCenter, Qt.AlignmentFlag.AlignTop);
		QWidget testingPage = new QWidget();
		testingPage.setLayout(layout);
		return testingPage;
	}

	/**
	 * Restores the preferences to a set of default values
	 */
	public void restoreDefaults()
	{
		Preferences.getInstance().setDefault();
		this.stackedLayout.removeWidget(this.preferences);
		this.preferences = createProgramPreferences();
		this.stackedLayout.addWidget(this.preferences);
		this.repaint();
	}

	/**
	 * Restores the preferences to the previous values
	 */
	public void restorePrevious()
	{
		this.close();
	}

	/**
	 * Update the temporary preferences with the values in the text fields. Possibly deprecated.
	 */
	@SuppressWarnings("unused")
	private void storeValues()
	{
		for(Entry<String, QLineEdit> i : this.textField.entrySet())
			this.updatePreference(i.getKey(), i.getValue().text());
		this.accept();
	}

	/**
	 * Set the preference represented by the specified key to the specified value, to be written on close.
	 * Used by the {@link PreferenceCheckBox}
	 * 
	 * @param s
	 *        A <code>String</code> representing the name of a preference
	 * @param b
	 *        A <code>boolean</code> representing the value of the preference
	 */
	public void updatePreference(String s, boolean b)
	{
		updatePreference(s, String.valueOf(b));
	}

	/**
	 * Set the preference represented by the specified key to the specified value, to be written on close.
	 * 
	 * @param k
	 *        A <code>String</code> representing the name of a preference
	 * @param v
	 *        A <code>String</code> representing the value of the preference
	 */
	public void updatePreference(String k, String v)
	{
		Preferences.getInstance().setValue(k, v);
	}
}
