/**
 * Preferences.java
 * Group 7, Object Oriented Software Engineering
 * October 14, 2009
 */
package edu.jhu.cs.oose.elevel;

import java.io.File;
import java.util.HashMap;
import java.util.Set;
import java.util.Map.Entry;
import com.trolltech.qt.QSysInfo;
import com.trolltech.qt.core.QDir;
import com.trolltech.qt.core.QSettings;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QWidget;

/**
 * Stores preferences for the E-Level application, implementing {@link QSettings}.
 * 
 * @author H. Parker Shelton, Jesse Yates
 */
public class Preferences
{
	/** Map of all the preferences and their values */
	HashMap<String, String> preferences;

	/** Company name used to identify the program */
	private static final String COMPANY_NAME = "OOSE";

	/** Product name used to identify the program */
	private static final String PRODUCT_NAME = "E_LEVEL";

	/** Singleton instance of the class */
	private static final Preferences INSTANCE = new Preferences();

	/**
	 * Get the singleton instance of this class
	 * 
	 * @return an instance of this class
	 */
	public static Preferences getInstance()
	{
		return INSTANCE;
	}

	/**
	 * Initializes a new Preferences. If an external file already exists, it is populated
	 * with the existing data. Otherwise, it is set to default preferences.
	 */
	private Preferences()
	{
		this.preferences = new HashMap<String, String>();
		readExternalSettings();

		if(this.preferences.size() == 0)
			setDefault();
	}

	/**
	 * Prompts the user for input if there is not a default directory
	 */
	public void changeExportLocation(QWidget parent, String text)
	{
		String initialPath = hasExportLocation() ? (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION") : QDir.homePath().toString();
		String path = QFileDialog.getExistingDirectory(parent, parent.tr(text), initialPath);
		if(!(path == null || path.equals("")))
		{
			// Assign a new path
			if(!path.endsWith("/"))
				path = path + "/";
			Preferences.getInstance().setValue("DEFAULT_EXPORT_LOCATION", path);
		}
		else // User canceled
		if(hasExportLocation())
			// Revert to previous location
			Preferences.getInstance().setValue("DEFAULT_EXPORT_LOCATION", initialPath);
		else
		{
			// Assign an arbitrary default location
			if(QSysInfo.windowsVersion() != 0) // Windows
				Preferences.getInstance().setValue("DEFAULT_EXPORT_LOCATION", QDir.homePath().toString() + "/My Documents/E-Level/");
			else if(QSysInfo.macVersion() != 0) // Mac OS X
				Preferences.getInstance().setValue("DEFAULT_EXPORT_LOCATION", QDir.homePath().toString() + "/Documents/E-Level/");
			else
				Preferences.getInstance().setValue("DEFAULT_EXPORT_LOCATION", QDir.homePath().toString() + "/Documents/E-Level/");
		}
		writeSettings();
	}

	/**
	 * Returns the <code>EntrySet</code> containing all the preferences
	 * 
	 * @return the <code>EntrySet</code> containing all the preferences
	 */
	public Set<Entry<String, String>> getEntrySet()
	{
		return this.preferences.entrySet();
	}

	/**
	 * Get the preference with the specified name
	 * 
	 * @param key
	 *        A <code>String</code> representing the name of the preference
	 * @return
	 *         An <code>Object</code> representing the value of the specified preference
	 */
	public String getValue(String key)
	{
		return this.preferences.get(key);
	}

	/**
	 * Returns whether or not the program has a default export location that exists on disk.
	 * 
	 * @return <code>true</code> if the program has a default export location and the directory exists on disk,
	 *         <code>false</code> otherwise.
	 */
	public boolean hasExportLocation()
	{
		if(Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION") == null || Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION").equals(""))
			return false;

		// Recreate a directory that does not currently exist on disk
		String loc = (String) Preferences.getInstance().getValue("DEFAULT_EXPORT_LOCATION");
		loc = loc.substring(0, loc.lastIndexOf('/')).substring(0, loc.lastIndexOf('/'));
		File f = new File(loc);
		if(!f.exists())
			// TODO - decide correct behavior
			// f.mkdir();
			return false;
		return true;
	}

	/**
	 * Read the settings in from the configuration file on disk
	 * 
	 * @see QSettings
	 */
	private void readExternalSettings()
	{
		QSettings settings = new QSettings(COMPANY_NAME, PRODUCT_NAME);

		settings.beginGroup("Preferences");

		for(String key : settings.allKeys())
			this.preferences.put(key, (String) settings.value(key, ""));

		settings.endGroup();
	}

	/**
	 * Set the default settings on the first run
	 */
	public void setDefault()
	{
		setValue("SHOW_SPLASH", "true");
		setValue("DEFAULT_EXPORT_LOCATION", "");
	}

	/**
	 * Set the preference represented by the specified key to the specified value, to be written on close
	 * 
	 * @param key
	 *        A <code>String</code> representing the name of a preference
	 * @param value
	 *        A <code>Object</code> representing the value of the preference
	 */
	public void setValue(String key, String value)
	{
		this.preferences.put(key, value);
	}

	/**
	 * Write all the current settings out to a file on disk
	 */
	public void writeSettings()
	{
		System.out.println("Writing Settings");
		QSettings settings = new QSettings(COMPANY_NAME, PRODUCT_NAME);
		settings.beginGroup("Preferences");

		for(Entry<String, String> e : this.preferences.entrySet())
			settings.setValue(e.getKey(), e.getValue());

		settings.endGroup();
	}
}
