/**
 * TestWindowGUI.java
 * Group 7, Object Oriented Software Engineering
 * October 10, 2009
 */
package edu.jhu.cs.oose.elevel;

import com.trolltech.qt.core.QRect;
import com.trolltech.qt.core.QSize;
import com.trolltech.qt.core.Qt.AlignmentFlag;
import com.trolltech.qt.gui.QAction;
import com.trolltech.qt.gui.QCloseEvent;
import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QGridLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QLineEdit;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QTextEdit;
import com.trolltech.qt.gui.QToolTip;
import com.trolltech.qt.gui.QWidget;
import com.trolltech.qt.gui.QPalette.ColorRole;

/**
 * GUI responsible for the view and controller for the underlying {@link TestWindow} model
 * 
 * @author Jesse Yates
 */
public class TestWindowGUI extends QDialog
{
	/** The underlying {@link TestWindow} model */
	private TestWindow testWindow;

	/** The QLayout for this component */
	private QGridLayout layout;

	/** The Widget to display */
	private QWidget display;

	/** The area of the <code>card</code> to display */
	private QWidget cardDisplay;

	/** QAction to exit the window */
	private QAction quit;

	/** QPushButtons for inputs */
	private QPushButton hint;
	private QPushButton submit;
	private QPushButton next;
	private QPushButton correct;
	private QPushButton incorrect;

	/** The input area for the answer */
	private QLineEdit answerInput;

	/** The associated label for answer input */
	private QLabel answerLabel;

	/** The current face displayed */
	private Face currentFace;

	/**
	 * Initializes a new <code>TestWindowGUI</code> with the specified parameters
	 * 
	 * @param test
	 *        A {@link TestWindowGUI} representing the underlying model
	 * @param parent
	 *        A {@link QWidget} representing the parent window
	 */
	public TestWindowGUI(TestWindow test, QWidget parent)
	{
		super(parent);

		this.testWindow = test;
		this.setBackgroundRole(ColorRole.Dark);
		this.setAutoFillBackground(true);
		this.setGeometry(parent.geometry());
		System.out.println("Testing card: " + this.testWindow.currentCard());

		this.layout = new QGridLayout();
		this.setLayout(this.layout);
		this.display = new QWidget();
		this.layout.addWidget(this.display);
		initalizeCard();

		this.addAction(this.quit);
		this.repaint();
	}

	/**
	 * Save the current deckStat to the {@link Deck} in the model for the test
	 * 
	 * @see com.trolltech.qt.gui.QDialog#closeEvent(com.trolltech.qt.gui.QCloseEvent)
	 */
	public void closeEvent(QCloseEvent event)
	{
		this.testWindow.finishTest();
	}

	/**
	 * Returns the {@link TestWindow} model this GUI is associated with
	 * 
	 * @return the {@link TestWindow} model this GUI is associated with
	 */
	public TestWindow getTestWindow()
	{
		return this.testWindow;
	}

	/**
	 * Initialize the parameters for the {@link Card}
	 */
	private void initalizeCard()
	{
		String s = Preferences.getInstance().getValue("TEST_FRONT_TO_BACK");
		if(s == null || s.equals("true"))
			setFace(Face.FRONT);
		else
			setFace(Face.BACK);
		setupActions();
		setupButtons();
		setupFront();
	}

	/**
	 * Get the next <code>card</code> or display the end of test prompt
	 */
	@SuppressWarnings("unused")
	private void nextCard()
	{
		if(!this.testWindow.nextCard())
		{
			QMessageBox.StandardButton msg = QMessageBox.warning(this, tr("Test"), "You have completed the test!\n" + "Retake the test?", new QMessageBox.StandardButtons(QMessageBox.StandardButton.Yes, QMessageBox.StandardButton.No));

			if(msg == QMessageBox.StandardButton.Yes)
				this.testWindow.newTest();
			else if(msg == QMessageBox.StandardButton.No)
			{
				this.close();
				return;
			}

		}

		nextFace();
		setupFront();
	}

	/**
	 * Determine the next face to show
	 */
	private void nextFace()
	{
		if(this.currentFace.equals(Face.FRONT))
			setFace(Face.BACK);
		else
			setFace(Face.FRONT);
	}

	/**
	 * Set the FACE information to the <code>Card</code> display area
	 * 
	 * @param f
	 *        the face of the card to be set
	 */
	private void setFace(Face f)
	{
		// TODO switch this from a QTextEdit to a generic that is set by some other object (probably a toolbar)

		if(f.equals(Face.FRONT))
		{
			this.cardDisplay = new QTextEdit(this.testWindow.currentCard().getQuestion(), this);
			this.currentFace = Face.FRONT;
		}
		else
		{
			this.cardDisplay = new QTextEdit(this.testWindow.currentCard().getAnswer(), this);
			this.currentFace = Face.BACK;
		}
	}

	/**
	 * Associates this GUI with the specified {@link TestWindow} model
	 * 
	 * @param t
	 *        the {@link TestWindow} model associated with this GUI
	 */
	public void setTestWindow(TestWindow t)
	{
		this.testWindow = t;
	}

	/**
	 * Setup the actions
	 */
	private void setupActions()
	{
		this.quit = new QAction(tr("&Quit"), this);
		this.quit.setShortcut(tr("Ctrl+Q"));
		this.quit.triggered.connect(this, "close()");
	}

	/**
	 * Setup the back of the <code>Card</code>
	 */
	private void setupBack()
	{
		this.display.hide();
		QWidget back = new QWidget();
		QGridLayout backLayout = new QGridLayout();
		back.setLayout(backLayout);
		backLayout.addWidget(this.cardDisplay, 0, 0, 6, 6);// , AlignmentFlag.AlignCenter);

		if(!this.testWindow.currentCard().options.get("Self-check Answers").booleanValue())
			this.answerInput.setText("Enter answer here");
		else
		{
			backLayout.addWidget(this.correct, 7, 2, 1, 1, AlignmentFlag.AlignCenter);
			backLayout.addWidget(this.incorrect, 7, 3, 1, 1, AlignmentFlag.AlignCenter);
		}
		backLayout.addWidget(this.next, 7, 5, 1, 1);

		this.layout.removeWidget(this.display);
		this.display = back;
		this.layout.addWidget(this.display, 0, 0);
		this.display.show();
	}

	/**
	 * Setup the buttons/inputs for the GUI
	 */
	private void setupButtons()
	{
		this.next = new QPushButton(tr("&Next Card"));
		this.next.clicked.connect(this, "nextCard()");

		this.correct = new QPushButton(tr("&Right!"));
		this.correct.clicked.connect(this.testWindow, "rightAnswer()");
		this.correct.clicked.connect(this, "nextCard()");
		this.correct.setDefault(true);
		this.correct.setAutoDefault(true);
		this.correct.setObjectName("root");
		this.correct.setStyleSheet("#root { background-color: green}");

		this.incorrect = new QPushButton(tr("&Wrong"));
		this.incorrect.clicked.connect(this.testWindow, "wrongAnswer()");
		this.incorrect.clicked.connect(this, "nextCard()");
		this.incorrect.setObjectName("root");
		this.incorrect.setStyleSheet("#root { background-color: red}");

		this.submit = new QPushButton(tr("&Submit"));
		this.submit.clicked.connect(this, "submit()");

		this.answerInput = new QLineEdit("Enter answer here.");

		this.answerLabel = new QLabel("Answer:");
		this.answerLabel.setBackgroundRole(ColorRole.Link);
		this.answerLabel.setBuddy(this.answerInput);

		// TODO decide if maybe we just want to use the ? button in the bar, rather than have a hint button
		// TODO make an icon for the hint
		this.hint = new QPushButton(tr("Hint"));
		this.hint.clicked.connect(this, "showHint()");
	}

	/**
	 * Setup the front of the <code>Card</code>
	 */
	private void setupFront()
	{
		this.display.hide();
		QWidget front = new QWidget();
		QGridLayout frontLayout = new QGridLayout();
		front.setLayout(frontLayout);

		frontLayout.addWidget(this.cardDisplay, 0, 0, 5, 6);// , AlignmentFlag.AlignCenter);
		frontLayout.addWidget(this.hint, 0, 5, 1, 1);
		frontLayout.setColumnMinimumWidth(6, 1);
		frontLayout.setRowMinimumHeight(6, 1);

		// System.out.println("Self-check Answers: " + this.card.options.get("Self-check Answers").booleanValue());
		if(!this.testWindow.currentCard().options.get("Self-check Answers").booleanValue())
		{
			this.answerInput.setText("Enter answer here");
			frontLayout.addWidget(this.answerLabel, 5, 0, 1, 1, AlignmentFlag.AlignCenter);
			frontLayout.addWidget(this.answerInput);// , 5, 1, 1, 4);// , AlignmentFlag.AlignCenter);
		}

		frontLayout.addWidget(this.submit, 7, 5, 1, 1);

		this.layout.removeWidget(this.display);
		this.display = front;
		this.layout.addWidget(this.display, 0, 0);
		this.display.show();
	}

	/**
	 * Shows the hint in a tooltip if clicked
	 */
	@SuppressWarnings("unused")
	private void showHint()
	{
		QToolTip.showText(this.geometry().center(), this.testWindow.currentCard().getHint(), this.hint, new QRect());
	}

	/**
	 * @see com.trolltech.qt.gui.QDialog#sizeHint()
	 */
	public QSize sizeHint()
	{
		return new QSize(200, 200);
	}

	/**
	 * Submits an answer and reveals the other face
	 */
	@SuppressWarnings("unused")
	private void submit()
	{
		System.out.println("Submitted!");
		nextFace();
		setupBack();
	}
}
